#-------------------------------------------------------------------------------------------------------------------------------------------------
#*************************************************************************************************
# Designed Jan. 2008 by Fredo6

# This software is provided as an example of using the Ruby interface to SketchUp.

# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted, provided that the above
# copyright notice appear in all copies.

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------------
# Name		:   LibVectorSelectorTool.rb
# Type		:   Utility Library as a Tool
# Description	:   Allow to select graphically a Vector, with and without length
# Menu Item	:   none
# Context Menu	:   none
# Usage		:   See Tutorial 
# Date		:   24 Jan 08
#-------------------------------------------------------------------------------------------------------------------------------------------------
#*************************************************************************************************

require 'sketchup.rb'
require 'libtraductor.rb'		# for language translation

module LibVectorSelectorTool

STATE_ORIGIN = 0
STATE_END = 1
STATE_EXECUTION = 2

MSG_Distance = "Distance |FR| Distance"
MSG_DefDistance = "Def. Distance |FR| Def. Distance"
MSG_Input_Origin = ["Select Origin of Vector",
                    "|FR| S\lectionner l'origine du vecteur"]
MSG_Input_End = ["Select End of Vector",
                 "|FR| S\lectionner l'extremit\ du vecteur"]
MSG_Input_Ctrl = ["(toggle Ctrl to use vector length as distance) - Default distance =",
                  "|FR| (Ctrl en bascule pour tenir compte de la distance) - distance par d\faut = "]
MSG_Input_Execution = ["Double Click or Enter to launch operation '%1' (or adjust angle value in VCB)",
                    "|FR| Double cliquez ou Enter pour ex\cuter l'op\ration '%1' (ou valeur de l'angle dans la VCB)"]
					
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Tool to select a plane, direction and angle - More or less mimic Skecthup Protractor
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
class VectorSelectorTool

def initialize(caller_class, distance, cursorpath=nil, opname="")
	Traductor.load_translation LibVectorSelectorTool, /MSG_/, binding, "@msg_"
	@caller = caller_class
	@cursorpath = cursorpath
	@opname = opname
	@idcursor = 0
	@distance_def = distance
	@ip_origin = Sketchup::InputPoint.new
	@ip_end = Sketchup::InputPoint.new
	@pt_origin = nil
	@pt_end = nil
	@use_length = false
		
	if (@cursorpath)
		f = Sketchup.find_support_file @cursorpath, "Plugins"
		@idcursor = UI::create_cursor f, 10, 10
	end	
end

def activate
	set_state STATE_ORIGIN
end

def deactivate(view)
	view.invalidate
end

def onSetCursor
	UI::set_cursor @idcursor if (@state == STATE_EXECUTION && @idcursor != 0)
end

def call_execute
	puts "method call"
	return unless @caller
	hmeth = @caller.method 'vector_selector_execute'
	if (hmeth == nil || hmeth.arity != 2)
		UI.beep
	else
		begin
			@vector = @pt_origin.vector_to @pt_end
			d = (@use_length) ? @pt_origin.distance(@pt_end) : nil
			puts "vector = #{@vector}"
			if hmeth.call @vector, d
				Sketchup.active_model.select_tool nil 
				return
			else
				set_state STATE_END
			end	
		rescue			
		end	
	end
end

def onLButtonDoubleClick(flags, x, y, view)
	if @state == STATE_EXECUTION
		call_execute
	else
		UI.beep
	end
end

def onKeyDown(key, rpt, flags, view)
	key = Traductor.check_key key, flags, false
	if (key == COPY_MODIFIER_KEY)
		@use_length = ! @use_length
		info_show
	end
end

def onKeyUp(key, rpt, flags, view)
	key = Traductor.check_key key, flags, true
	case key
	when 13			#Return key
		return UI.beep if @state < STATE_EXECUTION
		call_execute
	else
		return false
	end	
	true
end

#Control the 4 states of the tool
def set_state(state)
	@state = state
	@state = STATE_EXECUTION if (@state > STATE_EXECUTION)
	info_show
end

def onLButtonDown(flags, x, y, view)
	set_state @state + 1
end

#Handle Escape key
def onCancel(flag, view)
	return unless flag == 0
	return Sketchup.active_model.select_tool(nil) if (@state == STATE_ORIGIN)
	set_state @state - 1
end

#OnMouseMove method for Tool
def onMouseMove(flags, x, y, view)
	case @state	
	when STATE_ORIGIN		#input Origin of Vector
		@ip_origin.pick view, x, y
		view.tooltip = @ip_origin.tooltip
		@pt_origin = @ip_origin.position if @ip_origin.valid?
	when STATE_END			#input End of Vector
		@ip_end.pick view, x, y
		view.tooltip = @ip_end.tooltip
		@pt_end = @ip_end.position if @ip_end.valid? && @ip_end != @ip_origin
	end	
	view.invalidate
	info_show
end

#Draw method for tool
def draw(view)
	if (@state >= STATE_ORIGIN)
		view.draw_points @pt_origin, 10, 1, 'blue'
	end
	
	if (@state >= STATE_END && @pt_end)
		view.draw_points @pt_end, 15, 7, 'red'
		view.set_color_from_line @pt_origin, @pt_end
		view.draw_lines @pt_origin, @pt_end
	end
end

#display information in the Sketchup status bar
def info_show
	case @state
	when STATE_ORIGIN
		msg = @msg_Input_Origin
	when STATE_END
		msg = @msg_Input_End
	when STATE_EXECUTION
		msg = @msg_Input_Execution
	end
	d = @distance_def.to_l
	msg += " " + @msg_Input_Ctrl + " #{d}"
	Sketchup.set_status_text msg
	
	if (@state >= STATE_END && @use_length && @pt_end)
		Sketchup.set_status_text @msg_Distance, SB_VCB_LABEL
		Sketchup.set_status_text @pt_origin.distance(@pt_end).to_l, SB_VCB_VALUE
	else	
		Sketchup.set_status_text @msg_DefDistance, SB_VCB_LABEL
		Sketchup.set_status_text "[[#{d}]]", SB_VCB_VALUE
	end
end
	
end #Class VectorSelectorTool

end #Module LibVectorSelectorTool